#include <iostream>
#include <vector>
#include <string>

using namespace std;

class Dragon;
	
// The context class uses this class to call the 
// concrete strategy
class Strategy 
{
	public:
		virtual void fight(Dragon*) = 0; 
		virtual ~Strategy(){};
		virtual string getKind() = 0;
		virtual int injury(){return 0;};
};

class Dragon 
{
	public:
		Dragon() : strategy(0), health(10){};
		virtual ~Dragon(){delete strategy;};
		void setStrategy(Strategy* theStrategy)
		{
			strategy = theStrategy;
		};
		void fight(Dragon* d)
		{
			strategy->fight(d);
		};
		string getKind() 
		{
			return strategy -> getKind();
		};
		void receiveInjury()
		{
			health -= strategy -> injury();
		};
		bool isAlive(){return (health > 0);};
	private:
		Strategy* strategy;	
		int health;
};

// Implements the algorithm using the strategy interface
class Aggressive : public Strategy 
{
	 public:
		void fight(Dragon* d) 
		{
			cout << "Fight aggressively with " << d->getKind() << endl;
			d->receiveInjury();
		}
		string getKind()
		{
			return "an aggressive enemy";
		}			
};
 
class Defensive: public Strategy 
{

    public:
		void fight(Dragon* d) 
		{
			cout << "Fight defensively with " << d->getKind() << endl;
		}
		string getKind()
		{
			return "a defensive enemy";
		}
};
 
class Accurate: public Strategy 
{ 
	public:
		void fight(Dragon* d) 
		{
			cout << "Fight accurately with " << d->getKind() << endl;
		} 
		string getKind()
		{
			return "an accurate enemy";
		}
		int injury()
		{
			return 10;
		}
};
 

 
//StrategyExample test application
		
int pickStrategy()
{
	cout << "Specify strategy: " << endl;
	cout << "Accurate(0), Defensive(1), Aggressive(2): ";
	int choice;
	cin >> choice;
	return choice;
}

int main()
{
	enum{PLAYER_0, PLAYER_1, NOBODY};
	
	vector<Dragon*> dragon(2);
	dragon[PLAYER_0] = new Dragon();
	dragon[PLAYER_1] = new Dragon();
	
	vector<Strategy*> fightingStyle(3);
	fightingStyle[0] = new Accurate();
	fightingStyle[1] = new Defensive();
	fightingStyle[2] = new Aggressive();
	
	cout << "\nINITIALISE PLAYER 1\t";
	dragon[1]->setStrategy(fightingStyle[pickStrategy()]);
	
	int player = PLAYER_0;
	int winner = NOBODY;
	while (winner == NOBODY)
	{
		int enemy = (player+1)%2;
		if(dragon[player]->isAlive())
		{
			cout << "\nPLAYER " << player  << '\t';
			dragon[player]->setStrategy(fightingStyle[pickStrategy()]);
			dragon[player]->fight(dragon[enemy]);
		}
		else
		{
			winner = enemy;
		}
		player = enemy;
	}
	cout << "\n Player " << winner << " won!" << endl;
	return 0;
}
