//  x to the power of y 
#include <iostream>
#include <iomanip>
#include <cmath>

using namespace std;

// The context class uses this class to call the concrete strategy
class Strategy 
{
	public:
		virtual void execute(double a, double b) = 0; 
		virtual ~Strategy(){};
};
 
// Implements the algorithm using the strategy interface
class ConcreteStrategy_1: public Strategy 
{
	 public:
		void execute(double b, double p) 
		{
			double answer = b;
			cout << b;
			for (int i = 1; i < p; i++)
			{
				cout << " x " << b;
				answer *= b;
			}
			cout << fixed;
			if ((int)answer * 1000.0 == (int)(answer * 1000.0))
			{
				cout << setprecision(0);
			}
			else
			{
				cout << setprecision(4);
			}
			cout << " = " << answer << endl;
		} 
};
 
class ConcreteStrategy_2: public Strategy 
{

    public:
		void execute(double b, double p)
		{
			//only when (p >= 10 or p <= 1 or p is not an int)
			cout << b << " to the power of " << p;
			cout << fixed << setprecision(4);
			cout << " = " << pow(b,p) << endl;
		} 
};

// Configured with a ConcreteStrategy object and maintains a reference to a Strategy object
class Context 
{ 
	public:
		Context():strategy(0){};
		~Context(){strategy = 0;};
		void pickStrategy(double b, double p);
		void executeStrategy(double b, double p)
		{
			strategy->execute(b, p);
		};
	private:
		ConcreteStrategy_1 firstPrinciple;
		ConcreteStrategy_2 usePow;
		bool isInteger(double value)
		{
			return (int)value * 1000.0 == (int)(value * 1000.0);
		}
		Strategy* strategy;
};

void Context :: pickStrategy(double b, double p)
{
	if (p <10 && p > 1 && isInteger(p))
		strategy = &firstPrinciple;
	else
		strategy = &usePow;
}

int main()
{
	Context context;
	double base, power;
	//store default manipulation settings:
	ios_base :: fmtflags dflt = cout.flags(); 

	char choice = 'Y';
	while(choice == 'Y' || choice == 'y')
	{
		cout << "Enter a base number: ";
		cin >> base;
		cout << "To what power ?";
		cin >> power;
		cout.flags(dflt);  // Set manipulation back to default
		context.pickStrategy(base, power);
		context.executeStrategy(base, power);
		cout << endl << "More? (Y/N): ";
		cin >> choice;
		cout << endl;
	}
    return 0;
}
